<?php
/*
Le module de gestion des absences est à activer manuellement par l'administrateur dans l'interface du CDT (donc la seule synchronisation ne suffit pas).

Les tables à créer sont récupérées dans "administratrion/module_absence/module_absence_sql.php".

Pour voir des exemples d'insertions faites par le CDT :
   * insertion manuelle : administratrion/module_absence/ele_liste_affiche.php
   * insertion par CSV : administratrion/module_absence/import_absence_eleves_csv2.php

Un champ unique "uid_envole" est ajouté dans la table "ele_liste" afin d'avoir un critère valide d'unicité et de pouvoir tester l'existence d'un élève.

Principe actuel de la synchronisation :
 * pré-requis : pas de création des classes, elles doivent exister dans l'application (suite à l'importation en début d'année scolaire)
 * la synchronisation crée les élèves mais le module de gestion des absences est à activer manuellement par l'administrateur dans l'interface du CDT pour pouvoir être utilisé
 * la synchronisation est active par défaut (synchronisation avec suppression)
 * la synchronisation ne gère que les élèves dont la classe existe dans l'application
 * si le login d'un élève n'existe pas dans le CDT, l'élève est créé
 * si le login de l'élève existe, ses informations sont mises à jours (nom, prénom, classe)
 * si la suppression est active, elle concerne les élèves non retrouvés dans l'annuaire ou dont la classe n'existe plus dans le CDT
 * la suppression d'un élève entraîne la suppression de ses infos, de sa présence dans des groupes et de ses absences

Configuration de posh-profil : requêtes à ajouter dans le fichier "posh-profil/tmpl/posh-profil-update1.sql" du projet

 * vers la ligne 190 en modifiant l'ordre des autres pour avoir l'ordre alphabétique :

%if %%is_defined('activer_cdt') and %%activer_cdt == 'oui'
	INSERT INTO `admin_panel` (`admin_panel_id`, `admin_panel_label`, `admin_panel_url`, `admin_panel_order`, `admin_panel_group_id`) VALUES
	(22, 'CDT','synchro/cdt-admin.php',1,'20');
%end if

 * vers la ligne 290 :

-- Table d'administration de synchronisation du cahier de textes (CDT)
%if %%is_defined('activer_cdt') and %%activer_cdt == 'oui'
CREATE TABLE IF NOT EXISTS `admin_cdt` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `synchro_eleves` tinyint(1) NOT NULL,
  `del_eleves` tinyint(1) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8;

INSERT IGNORE INTO `admin_cdt` (`id`, `synchro_eleves`, `del_eleves`) VALUES (1, 1, 1);
%end if

Configuration du cron : instructions suivantes à ajouter dans le fichier "posh-profil/tmpl/posh-profil-synchronisation.sh" du projet

%if %%is_defined('activer_cdt') and %%activer_cdt == 'oui'
echo ""
echo "== SYNCHRONISATION CDT ====================================================================================================================="
php cdt-synchro.php -- key=$key
%end if
*/

/**
 * Classe de synchronisation des élèves dans le cahier de textes pour permettre la gestion des absences
 *
 * @author Christophe Deseure <christophe.deseure@ac-creteil.fr>
 */
class SynchroCDT
	{
	/**
	 * Configuration qui provient du fichier "include/envole.php"
	 *
	 * @var array
	 */
	private $aConfig = array();

	/**
	 * Pour repérer une exécution par cron et dans cas on ne produit pas d'affichage HTML
	 *
	 * @var boolean
	 */
	private $cron = false;

	/**
	 * Témoin pour savoir si la synchronisation est à faire
	 *
	 * @var boolean
	 */
	private $synchroEleves = false;

	/**
	 * Témoin pour savoir si il faut supprimer les élèves absents de l'annuaire lors de la synchronisation
	 *
	 * @var boolean
	 */
	private $suppressionEleves = false;

	/**
	 * Chemin relatif vers le dossier de l'application CDT
	 *
	 * @var string
	 */
	private $cdtPath = "";

	/**
	 * Objet pour les requêtes sql
	 *
	 * @var ps_DB
	 */
	private $oSql = "";

	/**
	 * Connexion à l'annuaire du scribe
	 *
	 * @var resource
	 */
	private $ldap = "";

	/**
	 * Message en cas d'erreur
	 *
	 * @var string
	 */
	private $messageErreur = null;

	/**
	 * Retourne true si aucune modif ou ajout n'a été faite sur les élèves
	 *
	 * @var boolean
	 */
	private $aucuneModifDbEleves = true;

	/**
	 * Liste des classes retrouvées dans le cdt
	 *
	 * @var array
	 */
	private $aClasse = array();

	/**
	 * Liste des élèves traitées (surtout utile si nettoyage à faire)
	 *
	 * @var array
	 */
	private $aEleve = array();

	/**
	 * Liste des tables à créer
	 *
	 * @var array
	 */
	private $aTable = array("ele_liste"=>"","ele_absent"=>"","ele_gic"=>"");

	/**
	 * Instanciation
	 *
	 * @param string $cdtPath
	 */
	public function __construct($cdtPath)
		{
		$this->cron = ($_SESSION['user_login']=="UserCron");
		$this->cdtPath = realpath(__DIR__."/".$cdtPath);
		}

	/**
	 * Opérations à faire en fin de traitement
	 */
	public function __destruct() { if($this->ldap) @ldap_close($this->ldap); }

	/**
	 * Initialisation des données bloquantes si invalides
	 *
	 * @param ps_DB  $oSql
	 * @param array  $aConfig configuration provenant du fichier "include/envole.php"
	 *
	 * @return boolean
	 */
	public function initialiser($oSql,$aConfig)
		{
		if($oSql instanceof ps_DB) $this->oSql = $oSql;
		else return $this->setErreur("initialisation incomplète : l'objet d'accès à la base de donnée est invalide");

		if(is_array($aConfig) && !empty($aConfig)) $this->aConfig = $aConfig;
		else return $this->setErreur("initialisation incomplète : configuration de posh-profil non récupérée");

		if(!$this->getConfigSynchro() || !$this->ldapConnexion()) return false; //message d'erreur géré par les méthodes appelées

		$this->oSql->dbname("cdt"); //obligatoirement après avoir récupéré la configuration dans la base "poshprofile" sélectionnée par défaut
		return true;
		}

	/**
	 * Lancer le traitement
	 *
	 * @return boolean
	 */
	public function executer()
		{
		$succes = true;
		if($this->synchroEleves)
		{
		 	$succes = ($this->verifierLesTables() && $this->parcourirLesEleves());
			$this->afficherTitre("Synthèse du travail de synchronisation de la base élèves");
			if ($this->aucuneModifDbEleves == true) $this->afficherSousTitre("La base des élèves n'a pas été modifiée.");
			else $this->afficherSousTitre("La base des élèves a été modifiée.");
		}
		else $this->afficherLigne("La synchronisation est désactivée.");
		return $succes;
		}

	/**
	 * Récupération des paramètres de la synchronisation
	 *
	 * @return boolean
	 */
	private function getConfigSynchro()
		{
		$requete = "SELECT * FROM `admin_cdt` WHERE `id`=1";
		$this->oSql->query($requete);
		if($this->oSql->next_record())
			{
			$this->synchroEleves = $this->oSql->f("synchro_eleves");
			$this->suppressionEleves = $this->oSql->f("del_eleves");
			return true;
			}
		else return $this->setErreur("paramètres de la synchronisation non récupérés");
		}

	/**
	 * Connexion au LDAP du scribe
	 *
	 * @return boolean
	 */
	private function ldapConnexion()
		{
		$aConfigObligatoire = array('LDAPserver','LDAPport','LDAPracine');
		foreach($aConfigObligatoire as $parametre)
			{
			if(!array_key_exists($parametre,$this->aConfig)) return $this->setErreur("connexion LDAP impossible : variable \"".$parametre."\" manquante");
			}

		$connexion = @ldap_connect($this->aConfig['LDAPserver'],$this->aConfig['LDAPport'] );
		if($connexion)
			{
			ldap_set_option($connexion,LDAP_OPT_PROTOCOL_VERSION,"3");
			$bind = @ldap_bind($connexion);
			if(!$bind) return $this->setErreur("erreur de connexion sur l'annuaire LDAP (ldap_bind)");
			}
		else return $this->setErreur("erreur de connexion sur l'annuaire LDAP (ldap_connect)");
		$this->ldap = $connexion;
		return true;
		}

	/**
	 * Tester si l'exécution est faite par le cron
	 *
	 * @return boolean
	 */
	public function synchroParCron() { return $this->cron; }

	/**
	 * Affichage générique
	 *
	 * @param string $labelHtml
	 * @param string $labelTexte
	 *
	 * @return string
	 */
	private function afficherLabel($labelHtml,$labelTexte) { echo $this->synchroParCron() ? $labelTexte."\n" : $labelHtml."\n"; }

	/**
	 * Affichage d'un titre
	 *
	 * @param string $label
	 */
	private function afficherTitre($label) { $this->afficherLabel("<br/><h1>".$label."</h1>","\n*** ".mb_strtoupper($label)." ***"); }

	/**
	 * Affichage d'un sous-titre
	 *
	 * @param string $label
	 */
	private function afficherSousTitre($label) { $this->afficherLabel("<br/><strong>".$label."</strong><br/>","\n>> ".$label); }

	/**
	 * Affichage d'un "sous-sous-titre"
	 *
	 * @param string $label
	 */
	private function afficherSousSousTitre($label) { $this->afficherLabel("<em>".$label."</em><br/>","* ".$label); }

	/**
	 * Affichage d'une ligne
	 *
	 * @param string $label
	 */
	private function afficherLigne($label) { $this->afficherLabel($label."<br/>",$label); }

	/**
	 * Fournir une message d'erreur
	 *
	 * @param string $erreur
	 *
	 * @return boolean toujours false
	 */
	private function setErreur($erreur="")
		{
		$this->messageErreur = $erreur;
		$this->afficherLabel("<p><b>[ERREUR] ".$erreur."</b></p>","\n>>>> ERREUR : ".$erreur);
		return false;
		}

	/**
	 * Récupérer le message d'erreur
	 *
	 * @return string
	 */
	public function getErreur() { return $this->messageErreur; }

	/**
	 * Installation des tables supplémentaires nécessaires à la gestion des absences
	 *
	 * Elle se fait en récupérant les requêtes dans le fichier source "cdt/administration/module_absence/module_absence_sql.php".
	 * Les requêtes sont systématiquement exécutées car il y a une condition d'existence à chaque fois (CREATE TABLE IF NOT EXISTS).
	 *
	 * @return boolean
	 */
	private function verifierLesTables()
		{
		$this->afficherTitre("vérification des tables nécessaires à la gestion des absences");

		$this->afficherSousTitre("Recherche du sql des tables concernées");
		$fichierSource = realpath($this->cdtPath."/administration/module_absence/module_absence_sql.php");
		if(!$fichierSource || !is_file($fichierSource)) return $this->setErreur("le fichier \"".$fichierSource."\" est introuvable");
		$contenu = file_get_contents($fichierSource);
		if(empty($contenu)) return $this->setErreur("le contenu du fichier \"".$fichierSource."\" n'a pas été récupéré");
		elseif(!preg_match_all('#query\s*=\s*"([^"]+)"\s*;#',$contenu,$aCapture,PREG_SET_ORDER)) return $this->setErreur("les requêtes SQL pour créer les tables sont introuvables");
		foreach($aCapture as $aInfo)
			{
			$requete = empty($aInfo[1]) ? "" : trim($aInfo[1]);
			if(preg_match('#^CREATE TABLE IF NOT EXISTS `(ele_[a-z]+)`#',$requete,$aSousCapture))
				{
				$table = $aSousCapture[1];
				if(array_key_exists($table,$this->aTable)) $this->aTable[$table] = utf8_encode($requete); //nécessaire car le script "module_absence_sql.php" est en ISO
				}
			}

		$this->afficherSousTitre("Création des tables si inexistantes");
		foreach($this->aTable as $table=>$requete)
			{
			if(empty($requete)) return $this->setErreur("le sql de création de la table \"".$table."\" non récupérée");
			elseif(!$this->oSql->query($requete)) return $this->setErreur("échec lors de la création d'une table : ".$this->oSql->error);
			}

		// https://dev-eole.ac-dijon.fr/issues/9791 : Suppression du UNIQUE(`uid_envole`) qui empèche la création si table non vide
		$this->afficherSousTitre("Ajout du champ spécifique \"uid_envole\" dans la table \"ele_liste\" si inexistant");
		$requete = "SELECT COUNT(*) AS `total` FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'ele_liste' AND COLUMN_NAME = 'uid_envole';";	
		if(!$this->oSql->query($requete) || !$this->oSql->next_record()) return $this->setErreur("impossible de vérifier l'existence du champ \"uid_envole\" : ".$this->oSql->error);
		$requete = "ALTER TABLE `ele_liste` ADD `uid_envole` VARCHAR(50) CHARACTER SET latin1 COLLATE latin1_general_ci NOT NULL DEFAULT '' AFTER `ID_ele`;";
		if($this->oSql->f("total")=="0" && !$this->oSql->query($requete)) return $this->setErreur("erreur pour ajouter le champ \"uid_envole\" : ".$this->oSql->error);

		// https://dev-eole.ac-dijon.fr/issues/9791 : 
		// Suite à la création du champ uid_envole si la table n'était pas vide, il faut supprimer les champs ayant un uid_envole vide (car sinon doublon)
		$requete = "DELETE FROM `ele_liste` WHERE `uid_envole` = ''";
		if(!$this->oSql->query($requete)) return $this->setErreur("erreur nettoyer la table si \"uid_envole=''\" : ".$this->oSql->error);

		return true;
		}

	/**
	 * Alimentation des élèves dans la base
	 *
	 * @return boolean
	 */
	private function parcourirLesEleves()
		{
		$this->afficherTitre("synchronisation des élèves de l'annuaire vers le cahier de textes");

		if(empty($this->ldap)) return $this->setErreur("impossible de parcourir les élèves : connexion LDAP invalide");

		if($this->suppressionEleves)
			{
			$this->afficherSousTitre("Lecture des élèves déjà présents dans le CDT");
			$requete = "SELECT `uid_envole` FROM `ele_liste`;";
			if($this->oSql->query($requete))
				{
				while($this->oSql->next_record())
					{
					$uid = $this->oSql->f("uid_envole");
					if(!empty($uid)) $this->aEleve[$uid] = false; //passage à true si retrouvé et donc mis à jour
					}
				}
			else $this->afficherLigne("ATTENTION : lecture invalide donc les élèves du CDT non présents dans l'annuaire ne seront pas supprimés"); //non bloquant
			}

		$this->afficherSousTitre("Création ou mise à jour des élèves dans le CDT");
		$recherche = ldap_search($this->ldap,$this->aConfig['LDAPracine'] ,"objectclass=Eleves",array("uid","sn","givenname","divcod"));
		$aEleve = ldap_get_entries($this->ldap,$recherche);
		$total = $aEleve['count'];
		$this->afficherLigne("nombre d'élèves à traiter : ".$total);
		for($i=0; $i<$total; $i++)
			{
			$aInfo = $aEleve[$i];
			$uid = $aInfo["uid"][0];
			$nom = $aInfo["sn"][0];
			$prenom = $aInfo["givenname"][0];
			$classe = $aInfo["divcod"][0];

			if(!$this->testerClasse($classe)) return false; //message d'erreur gérée par la méthode appelée
			elseif(!$this->existeClasse($classe)) $this->afficherLigne("[ERREUR] impossible de traiter l'élève \"".$prenom." ".$nom."\" : la classe \"".$classe."\" n'existe pas");
			elseif(!$this->ajouterEleve($uid,$nom,$prenom,$classe)) return false; //messages dont message d'erreur gérée par la méthode appelée
			}

		if($this->suppressionEleves)
			{
			$this->afficherTitre("Suppression des élèves non retrouvés dans l'annuaire ou dont la classe n'existe plus dans l'application");
			$suppressions = 0;
			foreach($this->aEleve as $uid=>$existe)
				{
				if($existe) continue;
				$getId = "SELECT `id_ele` FROM `ele_liste` WHERE `uid_envole`='".mysql_real_escape_string($uid)."'";
				$succes = $this->oSql->query("DELETE FROM `ele_absent` WHERE `eleve_ID` = (".$getId.");");
				$succes = $succes && $this->oSql->query("DELETE FROM `ele_gic` WHERE `id_ele` = (".$getId.");");
				$succes = $succes && $this->oSql->query("DELETE FROM `ele_liste` WHERE `uid_envole`='".mysql_real_escape_string($uid)."'");
				if(!$succes) $this->afficherLigne("[ERREUR] échec de la suppression de l'élève \"".$uid."\"");
				else
				{
				 ++$suppressions;
				$this->aucuneModifDbEleves = false;

				}
				}
			$this->afficherLigne("nombre d'élèves supprimés : ".$suppressions);
			}
		@ldap_free_result($recherche);
		return true;
		}

	/**
	 * Ajouter un élève s'il n'existe pas dans la base (la classe doit être vérifiée avant)
	 *
	 * @param string $uid
	 * @param string $nom
	 * @param string $prenom
	 * @param string $classe
	 *
	 * @return boolean
	 */
	private function ajouterEleve($uid,$nom,$prenom,$classe)
		{
		$test = $this->testerEleve($uid);
		if($test===false) return false; //message d'erreur gérée par la méthode appelée
		elseif($test===1) //élève qui existe déjà
			{
			// Avant de modifier la fiche de l'élève, on va déjà tester si il y a besoin de modifier
			$requete = "SELECT * FROM `ele_liste` WHERE `nom_ele` = '" . mysql_real_escape_string($nom) . "' AND `prenom_ele` = '" . mysql_real_escape_string($prenom) . "' AND `classe_ele` = '" . mysql_real_escape_string($classe) . "'";
			$this->oSql->query($requete);
			// On ne mets à jour que si ce n'est pas déjà fait
			if($this->oSql->next_record()) $this->afficherLigne($prenom." ".$nom." en ".$classe." : déjà à jour");
			else 
			{
				$this->aucuneModifDbEleves = false;
				$this->afficherLigne($prenom." ".$nom." en ".$classe." : mise à jour");
				$requete = sprintf
					(
					"UPDATE `ele_liste` SET `nom_ele` = '%s', `prenom_ele` = '%s', `classe_ele` = '%s' WHERE `uid_envole` = '%s';",
					mysql_real_escape_string($nom),
					mysql_real_escape_string($prenom),
					mysql_real_escape_string($classe),
					mysql_real_escape_string($uid)
					);
				if(!$this->oSql->query($requete)) return $this->setErreur("erreur pour mettre à jour l'élève \"".$prenom." ".$nom."\" : ".$this->oSql->error);
				
				}
			}
		else //nouvel élève
			{
			$this->aucuneModifDbEleves = false;
			$this->afficherLigne($prenom." ".$nom." en ".$classe." : ajout");
			$requete = sprintf
				(
				"INSERT INTO ele_liste (uid_envole,nom_ele,prenom_ele,classe_ele) VALUES ('%s','%s','%s','%s')",
				mysql_real_escape_string($uid),
				mysql_real_escape_string($nom),
				mysql_real_escape_string($prenom),
				mysql_real_escape_string($classe)
				);
			if(!$this->oSql->query($requete)) return $this->setErreur("impossible d'ajouter l'élève \"".$prenom." ".$nom."\" : ".$this->oSql->error);
			}
		$this->aEleve[$uid] = true;
		//if ($this->aucuneModifDbEleves == true) $this->afficherLigne("La base des élèves n'a pas été modifiée.");
		return true;
		}

	/**
	 * Tester la présence d'un élève dans la base
	 *
	 * @param string $uid
	 *
	 * @return mixed on renvoit false si problème, 0 si non trouvé et 1 si trouvé (max car uid_envole est une clé unique)
	 */
	private function testerEleve($uid)
		{
		$requete = "SELECT COUNT(*) AS `total` FROM `ele_liste` WHERE `uid_envole` = '".mysql_real_escape_string($uid)."';";
		if(!$this->oSql->query($requete) || !$this->oSql->next_record()) return $this->setErreur("impossible de vérifier la présence de l'élève \"".$uid."\" : ".$this->oSql->error);
		return intval($this->oSql->f("total"));
		}

	/**
	 * Tester l'existence d'une classe dans la base (fait qu'une fois)
	 *
	 * @param string $classe
	 *
	 * @return boolean
	 */
	private function testerClasse($classe)
		{
		if(!array_key_exists($classe,$this->aClasse))
			{
			$requete = "SELECT COUNT(*) AS `total` FROM `cdt_classe` WHERE `code_classe` = '".mysql_real_escape_string($classe)."';";
			if(!$this->oSql->query($requete) || !$this->oSql->next_record()) return $this->setErreur("impossible de vérifier l'existence de la classe \"".$classe."\" : ".$this->oSql->error);
			$this->aClasse[$classe] = (intval($this->oSql->f("total"))>0);
			}
		return true;
		}

	/**
	 * Vérifier l'existence d'une classe (elle doit d'abord avoir été testée)
	 *
	 * @param string $classe
	 *
	 * @return boolean
	 */
	private function existeClasse($classe) { return array_key_exists($classe,$this->aClasse) ? $this->aClasse[$classe] : false; }
	}

//error_reporting(E_ALL);
mb_internal_encoding("UTF-8");

// On vérifie si le php est executé en ligne de commande
if(isset($argv) && !empty($argv[2])) parse_str($argv[2], $param);
else $param = array("key"=>"");

$PATH = "../"; //nom $PATH imposé car utilisé par "styles/default/tpl/html.php" et "styles/default/tpl/frame.php"

require_once($PATH."include/include.php");
require_once($PATH."include/admin.php");

$oSynchro = new SynchroCDT("../../cdt");

if(!$oSynchro->synchroParCron())
	{
	include_once($PATH.$config["templatedir"]."/html.php");
	include_once($PATH.$config["templatedir"]."/frame.php");
	echo "</head><body><form name='Formulaire' enctype='multipart/form-data' action='' method='post'>";
	}

$oSql = new ps_DB();
$oSynchro->initialiser($oSql,$config) && $oSynchro->executer();

if(!$oSynchro->synchroParCron()) echo "</body></form></html>";
?>
